from functools import wraps

from .jwt_generator import create_access_token, create_refresh_token


def strawberry_auth_view(get_response):
    def add_cookies_response(request, response):
        if hasattr(request, "issueNewTokens") and request.issueNewTokens:
            response.set_cookie(
                key="JWT_ACCESS_TOKEN",
                value=create_access_token(request.clientID)
            )
            response.set_cookie(
                key="JWT_REFRESH_TOKEN",
                value=create_refresh_token(request.clientID)
            )
        if hasattr(request, "revokeTokens") and request.revokeTokens:
            response.delete_cookie("JWT_ACCESS_TOKEN")
            response.delete_cookie("JWT_REFRESH_TOKEN")
        return response

    @wraps(get_response)
    def wrapper(request, *args, **kwargs):
        request.jwt_cookie = True
        response = get_response(request, *args, **kwargs)
        return add_cookies_response(request, response)

    return wrapper



import jwt
from django.utils.timezone import datetime, timedelta

from .auth_settings import SECRET_KEY


def create_refresh_token(userID: int) -> str:
    refreshToken = jwt.encode(
        payload={
            "iat": datetime.utcnow(),
            "exp": datetime.utcnow() + timedelta(days=30),
        },
        key=SECRET_KEY,
        algorithm="HS256",
    )

    from models import RefreshTokens
    RefreshTokens.objects.create(
        user_id=userID,
        refreshToken=refreshToken,
    )

    return refreshToken


def create_access_token(userID: int) -> str:
    return jwt.encode(
        payload={
            "userID": userID,
            "iat": datetime.utcnow(),
            "exp": datetime.utcnow() + timedelta(minutes=1),
        },
        key=auth_settings.SECRET_KEY,
        algorithm="HS256",
    )



import jwt
from strawberry.extensions import Extension
from strawberry.types import ExecutionContext

from .auth_settings import SECRET_KEY


class JWTExtension(Extension):

    def __init__(self, execution_context: ExecutionContext):
        super().__init__(execution_context=execution_context)
        self.userID = None
        self.request = None
        self.response = None
        self.issueNewTokens = False
        self.revokeTokens = False

    def get_token_payload(self, tokenName):
        try:
            return jwt.decode(
                jwt=self.request.COOKIES[tokenName],
                key=SECRET_KEY,
                algorithms=["HS256"],  # todo: 1.make this dynamic 2. add nfb and leeway
            )
        except jwt.ExpiredSignatureError:
            return None
        except jwt.InvalidTokenError:
            return None

    def validate_access_token(self):
        if 'JWT_ACCESS_TOKEN' in self.request.COOKIES and self.request.COOKIES['JWT_ACCESS_TOKEN']:
            tokenPayload = self.get_token_payload(tokenName="JWT_ACCESS_TOKEN")
            if tokenPayload and 'userID' in tokenPayload:
                self.userID = tokenPayload['userID']
                return True
            return False

    def validate_refresh_token(self):
        if 'JWT_REFRESH_TOKEN' in self.request.COOKIES and self.request.COOKIES['JWT_REFRESH_TOKEN']:
            token_payload = self.get_token_payload(tokenName="JWT_REFRESH_TOKEN")
            if token_payload and token_payload is not None:
                try:
                    from models import RefreshTokens
                    refreshTokenObj = RefreshTokens.objects.get(refreshToken=self.request.COOKIES['JWT_REFRESH_TOKEN'])
                    if refreshTokenObj:
                        self.userID = refreshTokenObj.user.id
                        return True
                except RefreshTokens.DoesNotExist:
                    pass
            self.revokeTokens = True
            return False

    def on_request_start(self):
        self.request = self.execution_context.context["request"]
        if self.validate_access_token():
            pass
        elif self.validate_refresh_token():
            self.issueNewTokens = True

    def resolve(self, _next, root, info, *args, **kwargs):
        setattr(info.context, "userID", self.userID)
        if self.issueNewTokens:
            setattr(info.context.request, "issueNewTokens", self.issueNewTokens)
            setattr(info.context.request, "clientID", self.userID)
        if self.revokeTokens:
            setattr(info.context.request, "revokeTokens", self.revokeTokens)
        return _next(root, info, **kwargs)



from functools import wraps


def login_required(resolver):
    @wraps(resolver)
    def wrapper(parent, info, *args, **kwargs):
        userID = getattr(info.context, "userID", None)
        if userID is not None:
            return resolver(parent, info, *args, **kwargs)
        return None
    return wrapper




import codecs
from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.1.4'
DESCRIPTION = 'A JWT auth library based on Django and strawberry'
# Setting up
setup(
    name="strawberry_jwt_auth",
    version=VERSION,
    author="grvsh02 (Gaurav Sharma)",
    author_email="<gaurav021201@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['pyjwt', 'strawberry-graphql', 'django', 'phonenumbers'],
    keywords=['python', 'jwt', 'auth', 'strawberry', 'graphql', 'Authentication'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)